@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile$
                              -------------------
-    cvs         : $Id: main.c 453 2007-11-19 01:25:10Z martin $
+    cvs         : $Id$
     begin       : Mon Mar 01 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
@@ -20,8 +20,10 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/types.h>
-#include <sys/stat.h>
+#ifdef HAVE_SYS_WAIT_H
 #include <sys/wait.h>
+#endif
+#include <sys/stat.h>
 #include <fcntl.h>
 
 #ifdef HAVE_SIGNAL_H
@@ -46,6 +48,12 @@
     "This program is free software licensed under GPL.\n"\
     "See COPYING for details.\n"
 
+#ifdef OS_WIN32
+# include <windows.h>
+#
+# define usleep(x) Sleep((x/1000))
+#endif
+
 
 const GWEN_ARGS prg_args[]={
 {
@@ -140,7 +148,7 @@ const GWEN_ARGS prg_args[]={
 
 
 void usage(const char *name, const char *ustr) {
-  fprintf(stderr,
+  fprintf(stdout,
           I18N("KVKCard3 - A tool to read information from a German medical card.\n"
                "(c) 2007 Martin Preuss<martin@libchipcard.de>\n"
                "This library is free software; you can redistribute it and/or\n"
@@ -253,7 +261,7 @@ int main(int argc, char **argv) {
                      GWEN_ARGS_MODE_ALLOW_FREEPARAM,
                      prg_args,
                      db);
-  if (rv==-2) {
+  if (rv==GWEN_ARGS_RESULT_HELP) {
     GWEN_BUFFER *ubuf;
 
     ubuf=GWEN_Buffer_new(0, 256, 0, 1);
@@ -294,7 +302,12 @@ int main(int argc, char **argv) {
     rv=kvkRead(cl, db);
   }
   else if (strcasecmp(s, "daemon")==0) {
+#ifdef OS_WIN32
+    fprintf(stderr, "KVK daemon not yet supported on WIN32 platforms\n");
+    return RETURNVALUE_SETUP;
+#else
     rv=kvkDaemon(cl, db);
+#endif
   }
   else {
     fprintf(stderr, "Unknown command \"%s\"", s);
