@@ -1,7 +1,7 @@
 /***************************************************************************
  $RCSfile$
                              -------------------
-    cvs         : $Id: examplecard.c 378 2007-05-02 08:41:05Z martin $
+    cvs         : $Id$
     begin       : Mon Mar 01 2004
     copyright   : (C) 2004 by Martin Preuss
     email       : martin@libchipcard.de
@@ -210,4 +210,10 @@ int ExampleCard_GetExampleData(const LC_CARD *card){
 
 
 
+int test_type(TYPE_VISIBLE *v) {
+  return test_type(v);
+  return ((EXAMPLE_CARD*)v)->exampleData;
+}
+
+
 
